@@ -11,18 +11,9 @@ CREDENTIALS_SECRET_PATH = "gcp/skt-datahub/dataflow"
 
 
 def get_credentials():
-    import json
+    from skt.gcp_credentials import get_gcp_credentials
 
-    from google.oauth2 import service_account
-
-    from skt.vault_utils import get_secrets
-
-    key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
-    json_acct_info = json.loads(key)
-    credentials = service_account.Credentials.from_service_account_info(json_acct_info)
-    scoped_credentials = credentials.with_scopes(["https://www.googleapis.com/auth/cloud-platform"])
-
-    return scoped_credentials
+    return get_gcp_credentials()
 
 
 def set_default_credentials():
@@ -79,7 +70,7 @@ def _print_query_job_results(query_job):
             f"query: {query_job.query}\n"
             f"destination: {dest_str}\n"
             f"total_rows: {query_job.result().total_rows}\n"
-            f"slot_secs: {query_job.slot_millis/1000}\n"
+            f"slot_secs: {query_job.slot_millis / 1000}\n"
         )
     except Exception as e:
         print("Warning: exception on print statistics")
@@ -187,9 +178,11 @@ def _bq_table_to_pandas(table, project_id=PROJECT_ID):
 def _bq_cell_magic(line, query):
     import time
 
-    from google.cloud.bigquery.magics.magics import _cell_magic
+    from google.cloud.bigquery.magics.magics import _cell_magic, context
     from IPython.core import magic_arguments
 
+    context.credentials = get_credentials()
+
     start = time.time()
     args = magic_arguments.parse_argstring(_cell_magic, line)
 
@@ -236,7 +229,6 @@ def _is_ipython():
 def set_gcp_credentials():
     import os
     import tempfile
-
     from skt.vault_utils import get_secrets
 
     key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
@@ -255,19 +247,38 @@ def load_bigquery_ipython_magic():
     if _is_ipython():
         from IPython import get_ipython
 
-        set_gcp_credentials()
         _load_bq_ipython_extension(get_ipython())
     else:
         raise Exception("Cannot import bigquery magic. Because execution is not on ipython.")
 
 
 def get_bigquery_client(credentials=None, project_id=PROJECT_ID, location=LOCATION):
+    import json
+    import os
     from google.cloud import bigquery
 
     if credentials is None:
         credentials = get_credentials()
 
-    return bigquery.Client(credentials=credentials, project=project_id, location=location)
+    job_config_labels = {
+        "nes_client_id": str(os.environ.get("CLIENT_ID", "")),
+        "jupyterhub_user": str(os.environ.get("JUPYTERHUB_USER", "")),
+    }
+
+    nes_client_context = os.environ.get("CLIENT_CONTEXT")
+    if nes_client_context:
+        nes_client_context_dict = json.loads(nes_client_context)
+        job_config_labels.update(nes_client_context_dict)
+
+    for k, v in job_config_labels.items():
+        job_config_labels.update({k: v.lower().replace(".", "_") if v else v})
+
+    return bigquery.Client(
+        credentials=credentials,
+        project=project_id,
+        location=location,
+        default_query_job_config=QueryJobConfig(labels=job_config_labels),
+    )
 
 
 def bq_table_exists(table, project_id=PROJECT_ID):
@@ -310,14 +321,16 @@ def _bq_table_to_df(
 ):
     import base64
 
-    from skt.vault_utils import get_secrets
+    from skt.gcp_credentials import get_gcp_credentials_json_string
 
     if not spark_session:
         spark_session = get_spark()
     spark_session.conf.set("spark.sql.execution.arrow.enabled", "false")
-    key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
+
+    key = get_gcp_credentials_json_string()
     df = (
         spark_session.read.format("bigquery")
+        .option("parentProject", project_id)
         .option("project", project_id)
         .option("table", f"{project_id}:{dataset}.{table_name}")
         .option("credentials", base64.b64encode(key.encode()).decode())
@@ -380,12 +393,13 @@ def _df_to_bq_table(
 ):
     import base64
 
-    from skt.vault_utils import get_secrets
+    from skt.gcp_credentials import get_gcp_credentials_json_string
 
-    key = get_secrets(CREDENTIALS_SECRET_PATH)["config"]
+    key = get_gcp_credentials_json_string()
     table = f"{dataset}.{table_name}${partition}" if partition else f"{dataset}.{table_name}"
     df = (
         df.write.format("bigquery")
+        .option("parentProject", project_id)
         .option("project", project_id)
         .option("credentials", base64.b64encode(key.encode()).decode())
         .option("table", table)
