@@ -1,5 +1,14 @@
+from typing import TYPE_CHECKING, Optional, Sequence, Union
+
 from skt.vault_utils import get_secrets
 
+if TYPE_CHECKING:
+    import pandas as pd
+    from hmsclient import hmsclient
+    from pyarrow import fs
+    from pyspark import SparkConf, SparkContext
+    from pyspark.sql import SparkSession
+
 
 def get_hms():
     from hmsclient import hmsclient
@@ -51,152 +60,139 @@ def get_pkl_from_hdfs(pkl_path):
     return pkl_object
 
 
-def get_spark(scale=0, queue=None, jars=None):
+def is_in_notebook():
+    try:
+        from IPython import get_ipython
+
+        if get_ipython().__class__.__name__ == "ZMQInteractiveShell":
+            return True
+        else:
+            return False
+    except (NameError, ModuleNotFoundError):
+        return False
+
+
+def _spark_context_to_repr_html(sc: "SparkContext") -> str:
+    import os
+
+    if os.environ.get("JUPYTERHUB_USER"):
+        ui_web_url = sc.uiWebUrl
+
+    else:
+        history_server_url = os.environ.get("SPARK_HISTORY_SERVER_URL", "http://spark.yks.sktai.io")
+        ui_web_url = f"{history_server_url}/history/{sc.applicationId}"
+
+    return f"""
+        <div>
+            <p><b>SparkContext</b></p>
+
+            <p><a href="{ui_web_url}">Spark UI</a></p>
+
+            <dl>
+              <dt>Version</dt>
+                <dd><code>v{sc.version}</code></dd>
+              <dt>Master</dt>
+                <dd><code>{sc.master}</code></dd>
+              <dt>AppName</dt>
+                <dd><code>{sc.appName}</code></dd>
+            </dl>
+        </div>
+        """
+
+
+def get_spark(
+    scale: int = 0,
+    queue: Optional[str] = None,
+    jars: Optional[Union[Sequence[str], str]] = None,
+    conf: Optional["SparkConf"] = None,
+) -> "SparkSession":
+    import json
     import os
     import uuid
-    import tempfile
-    from pyspark.sql import SparkSession
-    from skt.vault_utils import get_secrets
-    from pyspark import version as spark_version
+    import warnings
 
-    is_spark_3 = spark_version.__version__ >= "3.0.0"
+    from pyspark.sql import SparkSession
 
     tmp_uuid = str(uuid.uuid4())
     app_name = f"skt-{os.environ.get('USER', 'default')}-{tmp_uuid}"
 
-    key = get_secrets("gcp/sktaic-datahub/dataflow")["config"]
-    key_file_name = tempfile.mkstemp()[1]
-    with open(key_file_name, "wb") as key_file:
-        key_file.write(key.encode())
-        os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = key_file.name
+    if queue:
+        warnings.warn("Queue is unsupported not implemented yet")
 
-    if not queue:
-        if "JUPYTERHUB_USER" in os.environ:
-            queue = "dmig_eda"
-        else:
-            queue = "airflow_job"
+    session_builder = SparkSession.builder
 
-    bigquery_jars = (
-        "hdfs:///jars/spark-bigquery-with-dependencies_2.12-0.24.2.jar"
-        if is_spark_3
-        else "hdfs:///jars/spark-bigquery-with-dependencies_2.11-0.17.3.jar"
+    if conf:
+        session_builder.config(conf=conf)
+
+    session_builder.appName(app_name)
+
+    if jars is None:
+        pass
+    elif isinstance(jars, str):
+        session_builder.config("spark.jars", jars)
+    else:
+        session_builder.config("spark.jars", ",".join(jars))
+
+    driver_cores = 1
+    executor_cores = 1
+
+    if scale == 0:
+        driver_memory_GB = 8
+        driver_max_result_size_GB = 6
+        executor_memory_GB = 8
+        executor_instances = 8
+    elif scale >= 1 and scale <= 4:
+        driver_memory_GB = scale * 8
+        driver_max_result_size_GB = scale * 4
+        executor_memory_GB = scale * 3
+        executor_instances = scale * 8
+    elif scale >= 5 and scale <= 8:
+        driver_memory_GB = 8
+        driver_max_result_size_GB = 8
+        executor_memory_GB = min(2**scale, 200)
+        executor_instances = 32
+    else:
+        raise ValueError("Invalid scale")
+
+    task_group = [
+        {
+            "name": "executor",
+            "minMember": int(executor_instances),
+            "minResource": {
+                "cpu": str(executor_cores),
+                "memory": f"{executor_memory_GB * 1.1}Gi",  # JVM Overhead
+            },
+            "nodeSelector": {
+                "role/spark": None,
+                "zone": "cpu.pool",
+            },
+            "tolerations": [],
+            "affinity": {},
+            "topologySpreadConstraints": [],
+        }
+    ]
+
+    spark = (
+        session_builder.config("spark.driver.cores", str(driver_cores))
+        .config("spark.driver.memory", f"{driver_memory_GB}g")
+        .config("spark.driver.maxResultSize", f"{driver_max_result_size_GB}g")
+        .config("spark.executor.cores", str(executor_cores))
+        .config("spark.executor.memory", f"{executor_memory_GB}g")
+        .config("spark.executor.instances", str(executor_instances))
+        .config("spark.kubernetes.executor.annotation.yunikorn.apache.org/task-group-name", "executor")
+        .config("spark.kubernetes.executor.annotation.yunikorn.apache.org/task-groups", json.dumps(task_group))
+        .config(
+            "spark.kubernetes.executor.annotation.yunikorn.apache.org/schedulingPolicyParameters",
+            "placeholderTimeoutInSeconds=60 gangSchedulingStyle=Hard",
+        )
+        .getOrCreate()
     )
 
-    spark_jars = ",".join([bigquery_jars, jars]) if jars else bigquery_jars
+    if is_in_notebook():
+        from IPython.display import HTML, display
 
-    arrow_enabled = "spark.sql.execution.arrow.pyspark.enabled" if is_spark_3 else "spark.sql.execution.arrow.enabled"
+        display(HTML(_spark_context_to_repr_html(spark.sparkContext)))
 
-    arrow_pre_ipc_format = "0" if is_spark_3 else "1"
-    os.environ["ARROW_PRE_0_15_IPC_FORMAT"] = arrow_pre_ipc_format
-
-    if queue == "nrt":
-        spark = (
-            SparkSession.builder.config("spark.app.name", app_name)
-            .config("spark.driver.memory", "6g")
-            .config("spark.executor.memory", "4g")
-            .config("spark.driver.maxResultSize", "6g")
-            .config("spark.rpc.message.maxSize", "1024")
-            .config("spark.executor.core", "4")
-            .config("spark.executor.instances", "32")
-            .config("spark.yarn.queue", queue)
-            .config("spark.ui.enabled", "false")
-            .config("spark.port.maxRetries", "128")
-            .config("spark.executorEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-            .config("spark.yarn.appMasterEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-            .config(
-                "spark.jars",
-                spark_jars,
-            )
-            .enableHiveSupport()
-            .getOrCreate()
-        )
-        spark.conf.set(arrow_enabled, "true")
-        return spark
-
-    if scale in [1, 2, 3, 4]:
-        spark = (
-            SparkSession.builder.config("spark.app.name", app_name)
-            .config("spark.driver.memory", f"{scale*8}g")
-            .config("spark.executor.memory", f"{scale*3}g")
-            .config("spark.executor.instances", f"{scale*8}")
-            .config("spark.driver.maxResultSize", f"{scale*4}g")
-            .config("spark.rpc.message.maxSize", "1024")
-            .config("spark.yarn.queue", queue)
-            .config("spark.ui.enabled", "false")
-            .config("spark.port.maxRetries", "128")
-            .config("spark.executorEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-            .config("spark.yarn.appMasterEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-            .config(
-                "spark.jars",
-                spark_jars,
-            )
-            .enableHiveSupport()
-            .getOrCreate()
-        )
-    elif scale in [5, 6, 7, 8]:
-        spark = (
-            SparkSession.builder.config("spark.app.name", app_name)
-            .config("spark.driver.memory", "8g")
-            .config("spark.executor.memory", f"{2 ** scale}g")
-            .config("spark.executor.instances", "32")
-            .config("spark.driver.maxResultSize", "8g")
-            .config("spark.rpc.message.maxSize", "1024")
-            .config("spark.yarn.queue", queue)
-            .config("spark.ui.enabled", "false")
-            .config("spark.port.maxRetries", "128")
-            .config("spark.executorEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-            .config("spark.yarn.appMasterEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-            .config(
-                "spark.jars",
-                spark_jars,
-            )
-            .enableHiveSupport()
-            .getOrCreate()
-        )
-    else:
-        if is_spark_3:
-            spark = (
-                SparkSession.builder.config("spark.app.name", app_name)
-                .config("spark.driver.memory", "8g")
-                .config("spark.executor.memory", "8g")
-                .config("spark.executor.instances", "8")
-                .config("spark.driver.maxResultSize", "6g")
-                .config("spark.rpc.message.maxSize", "1024")
-                .config("spark.yarn.queue", queue)
-                .config("spark.ui.enabled", "false")
-                .config("spark.port.maxRetries", "128")
-                .config("spark.executorEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-                .config("spark.yarn.appMasterEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-                .config(
-                    "spark.jars",
-                    spark_jars,
-                )
-                .enableHiveSupport()
-                .getOrCreate()
-            )
-        else:
-            spark = (
-                SparkSession.builder.config("spark.app.name", app_name)
-                .config("spark.driver.memory", "6g")
-                .config("spark.executor.memory", "8g")
-                .config("spark.shuffle.service.enabled", "true")
-                .config("spark.dynamicAllocation.enabled", "true")
-                .config("spark.dynamicAllocation.maxExecutors", "200")
-                .config("spark.driver.maxResultSize", "6g")
-                .config("spark.rpc.message.maxSize", "1024")
-                .config("spark.yarn.queue", queue)
-                .config("spark.ui.enabled", "false")
-                .config("spark.port.maxRetries", "128")
-                .config("spark.executorEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-                .config("spark.yarn.appMasterEnv.ARROW_PRE_0_15_IPC_FORMAT", arrow_pre_ipc_format)
-                .config(
-                    "spark.jars",
-                    spark_jars,
-                )
-                .enableHiveSupport()
-                .getOrCreate()
-            )
-    spark.conf.set(arrow_enabled, "true")
     return spark
 
 
@@ -273,7 +269,7 @@ def slack_send(
     icon_emoji=":large_blue_circle:",
     blocks=None,
     dataframe=False,
-    adot=False
+    adot=False,
 ):
     import requests
     from skt.vault_utils import get_secrets
@@ -283,7 +279,11 @@ def slack_send(
 
         text = "```" + tabulate(text, tablefmt="simple", headers="keys") + "```"
 
-    token = get_secrets("airflow_k8s/adot_slack/slack_alarmbot_token")["token"] if adot else get_secrets("slack")["bot_token"]["airflow"]
+    token = (
+        get_secrets("airflow_k8s/adot_slack/slack_alarmbot_token")["token"]
+        if adot
+        else get_secrets("slack")["bot_token"]["airflow"]
+    )
     proxy = get_secrets("proxy")["proxy"]
     proxies = {
         "http": proxy,
